"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types = require("@babel/types");
const NodeGenerator_1 = require("../helpers/NodeGenerator");
/**
 * Represents a mutator which can remove the conditional clause from statements.
 */
class ConditionalExpressionMutator {
    constructor() {
        this.validOperators = ['!=', '!==', '&&', '<', '<=', '==', '===', '>', '>=', '||'];
        this.name = 'ConditionalExpression';
    }
    hasValidParent(node) {
        return (!node.parent ||
            !(types.isForStatement(node.parent) ||
                types.isWhileStatement(node.parent) ||
                types.isIfStatement(node.parent) ||
                types.isDoWhileStatement(node.parent)));
    }
    isValidOperator(operator) {
        return this.validOperators.includes(operator);
    }
    mutate(node, copy) {
        if ((types.isBinaryExpression(node) || types.isLogicalExpression(node)) && this.hasValidParent(node) && this.isValidOperator(node.operator)) {
            return [NodeGenerator_1.NodeGenerator.createBooleanLiteralNode(node, false), NodeGenerator_1.NodeGenerator.createBooleanLiteralNode(node, true)];
        }
        else if (types.isDoWhileStatement(node) || types.isWhileStatement(node)) {
            return [NodeGenerator_1.NodeGenerator.createBooleanLiteralNode(node.test, false)];
        }
        else if (types.isForStatement(node)) {
            if (!node.test) {
                const mutatedNode = copy(node);
                mutatedNode.test = NodeGenerator_1.NodeGenerator.createBooleanLiteralNode(node, false);
                return [mutatedNode];
            }
            else {
                return [NodeGenerator_1.NodeGenerator.createBooleanLiteralNode(node.test, false)];
            }
        }
        else if (types.isIfStatement(node)) {
            return [NodeGenerator_1.NodeGenerator.createBooleanLiteralNode(node.test, false), NodeGenerator_1.NodeGenerator.createBooleanLiteralNode(node.test, true)];
        }
        else if (types.isSwitchCase(node) &&
            // if not a fallthrough case
            node.consequent.length > 0) {
            const mutatedNode = copy(node);
            mutatedNode.consequent = [];
            return [mutatedNode];
        }
        return [];
    }
}
exports.default = ConditionalExpressionMutator;
//# sourceMappingURL=ConditionalExpressionMutator.js.map